package com.swtjface.flowchart.editpart;

import java.beans.*;
import java.util.*;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.geometry.*;
import org.eclipse.gef.*;
import org.eclipse.gef.requests.DropRequest;
import com.swtjface.flowchart.figures.*;
import com.swtjface.flowchart.model.*;
import com.swtjface.flowchart.policies.*;

abstract public class ActivityPart
  extends org.eclipse.gef.editparts.AbstractGraphicalEditPart
  implements NodeEditPart, PropertyChangeListener
{
  public void propertyChange(PropertyChangeEvent evt)
  {
    String prop = evt.getPropertyName();
    if (Activity.CHILD.equals(prop))
      refreshChildren();
    else if (Activity.TARGETS.equals(prop))
      refreshTargetConnections();
    else if (Activity.SOURCES.equals(prop))
      refreshSourceConnections();
    else if (Activity.SIZE.equals(prop) ||
      Activity.LOC.equals(prop) ||
      Activity.NAME.equals(prop))
        refreshVisuals();
  }

  protected void refreshVisuals()
  {
    getActivityFigure().setName(getActivity().getName());
    Point loc = getActivity().getLocation();
    Dimension size = getActivity().getSize();
    Rectangle r = new Rectangle(loc ,size);
    ((GraphicalEditPart) getParent()).setLayoutConstraint
      (this, getFigure(), r);
  }

  public void activate()
  {
    if (isActive())
      return;
    super.activate();
    getActivity().addPropertyChangeListener(this);
  }

  public void deactivate()
  {
    if (!isActive())
      return;
    super.deactivate();
    getActivity().removePropertyChangeListener(this);
  }

  protected void createEditPolicies()
  {
    installEditPolicy(EditPolicy.COMPONENT_ROLE, new ComponentPolicy());
    installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new NodePolicy());
  }

  public ConnectionAnchor getSourceConnectionAnchor
    (ConnectionEditPart connEditPart)
  {
    Path Path = (Path) connEditPart.getModel();
    return getActivityFigure().
    getSourceConnectionAnchor(Path.getSourceName());
  }
  
  public ConnectionAnchor getSourceConnectionAnchor
    (Request request)
  {
    Point pt = new Point(((DropRequest)request).getLocation());
    return getActivityFigure().getSourceConnectionAnchorAt(pt);
  }

  public ConnectionAnchor getTargetConnectionAnchor
    (ConnectionEditPart connEditPart)
  {
    Path Path = (Path) connEditPart.getModel();
    return getActivityFigure().
    getTargetConnectionAnchor(Path.getTargetName());
  }

  public ConnectionAnchor getTargetConnectionAnchor
    (Request request)
  {
    Point pt = new Point(((DropRequest)request).getLocation());
    return getActivityFigure().getTargetConnectionAnchorAt(pt);
  }

  public String mapSourceConnectionAnchorToName
  (ConnectionAnchor c)
  {
    return getActivityFigure().getSourceAnchorName(c);
  }

  public String mapTargetConnectionAnchorToName
  (ConnectionAnchor c)
  {
    return getActivityFigure().getTargetAnchorName(c);
  }

  protected ActivityFigure getActivityFigure()
  {
    return (ActivityFigure) getFigure();
  }

  protected Activity getActivity()
  {
    return (Activity)getModel();
  }

  protected List getModelSourceConnections()
  {
    return getActivity().getSourceConnections();
  }

  protected List getModelTargetConnections()
  {
    return getActivity().getTargetConnections();
  }
}